/*global define,window,folder*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/mediagallery/filemanager/folder-view',
    'de_epages/mediagallery/filemanager/folder-model',

    '$ready!'
], function (QUnit, $, FolderView, FolderModel) {
    'use strict';

    var setup = function () {
        var filemanager = $('<ul class="filemanager" />').appendTo('#qunit-fixture'),
            model = new FolderModel({
                "files": [],
                "children": [],
                "metadata": {
                    "displayname": "neu",
                    "type": "MediaGallery",
                    "path": "/neu"
                },
                "data": "neu",
                "state": null
            });

        window.folder = new FolderView({model: model});

        folder.on('openFolder', function (e) {
            QUnit.strictEqual(
                e.displayname,
                "neu",
                'Trigger "openFolder" event on dblclick'
            );
        });

        filemanager.append(folder.render().el);

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer').length,
            1,
            'Create folder view and append it to file list'
        );
    },
    teardown = function () {
        $('.ep-mediagallery-folderContainer').remove();

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer').length,
            0,
            'Remove folder view from file list'
        );
    };

    QUnit.asyncTest('view methods', 7 + 2, function () {
        setup();
        $.each(['initialize', 'render', 'edit', 'onKeyup', 'save', 'directoryAlreadyExists', 'openFolder'], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(folder[method]),
                true,
                method + ' is a method of folder view'
            );
        });

        teardown();
        QUnit.start();
    });

    QUnit.asyncTest('view functionality', 3 + 3, function () {
        setup();
        var e = $.Event('keyup');

        folder.edit();

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer input').length,
            1,
            'Execute edit method - now an input is available'
        );

        e.which = 13;
        e.target = $('.ep-mediagallery-folderContainer input')[0];
        e.target.value = "neu";

        folder.onKeyup(e);
        folder.save(e);

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer input').length,
            0,
            'Execute save method - rerender folder view'
        );

        folder.edit();

        var evt = $.Event('focusout');
        evt.target = $('.ep-mediagallery-folderContainer input')[0];
        evt.target.value = "newFolderName";

        folder.save(evt);
        folder.directoryAlreadyExists();

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer input').hasClass('DialogError'),
            true,
            'Folder name already exists'
        );

        folder.render();
        $('.ep-mediagallery-folderContainer').trigger('dblclick');
        teardown();
        QUnit.start();
    });

});