/**
 * Add date selection from calendar handling for input elements.
 *
 * The `.uiDatepicker()` method add validate handling to a input elements.
 *
 * The `.uiDatepicker()` method is a validation input also and using the validation type ''date'' from
 * the ep.validate module. Have a look at the additional options, some options are only available for
 * a special validation types.
 *
 * ### Examples
 * Apply .uiDatepicker() to input element named date.
 *
 * JavaScript:
 *
 *     ep(':input[name=date]')
 *         .uiDatepicker({
 *             type:       'date',
 *             format:     'd',
 *             time:       false
 *         });
 *
 * HTML:
 *
 *     <form>
 *       <input type="text" name="date" />
 *     </form>
 *
 *
 * @class jQuery.ui.uiDatepicker
 * @extends jQuery.ui.uiValidate
 *
 * @uses ep
 * @uses ep.date
 * @uses ep.sprite
 * @uses ep.fn.busy
 * @uses ep.ui.spinner
 * @uses jQuery.tmpl
 * @since 6.11.0
 */

/**
 * Call up a previously attached date picker.
 *
 * @method show
 * @member jQuery.ui.uiDatepicker
 *
 * @since 6.11.0
 */

/**
 * Close a previously opened date picker.
 *
 * @method hide
 * @member jQuery.ui.uiDatepicker
 *
 * @since 6.11.0
 */

/**
 * @cfg {String} type The value respectively validation type must be 'date'.
 */

/**
 * @cfg {String} format A date formatter [look at the jQuery.i18n plugin].
 */

/**
 * @cfg {Integer} min A local timestamp (not UTC) as minimum valid date.
 */

/**
 * @cfg {Integer} max A local timestamp (not UTC) as maximum valid date.
 */

/**
 * @cfg {String} showOn A handle indication, how to show the datepicker. Supported handles: 'focus', 'button', 'both' (button and focus) or 'inline'.
 */

/**
 * @cfg {Integer,String} showOn A duration for show and hide effect. [look at jQuery.show() or jQuery.hide() for supported durations].
 */

/**
 * @cfg {String} show One off the supported show effects: 'show', 'fadeIn' or 'slideIn'.
 */

/**
 * @cfg {String} hide One off the supported hide effects: 'hide', 'fadeOut' or 'slideOut'.
 */

/**
 * @cfg {Array} weekDaysAvailable An array of 7 weekdays with indications wether a day is available. Week starts on sunday.
 */

/**
 * @cfg {Integer} numberOfMonths Set how many months to show at once.
 */

/**
 * @cfg {Boolean} showOtherMonth Display dates in other months (non-selectable) at the start or end of the current month. To make these days selectable use selectOtherMonths.
 */

/**
 * @cfg {Boolean} selectOtherMonth When true days in other months shown before or after the current month are selectable. This only applies if showOtherMonths is also true.
 */

/**
 * @cfg {Boolean} changeMonth Allows you to change the month by selecting from a drop-down list. You can enable this feature by setting the attribute to true.
 */

/**
 * @cfg {Boolean} changeYear Allows you to change the year by selecting from a drop-down list. You can enable this feature by setting the attribute to true. Use the yearRange option to control which years are made available for selection.
 */

/**
 * @cfg {Boolean} time Allows you to select time also. Requires a format which support time.
 */

/**
 * @cfg {Boolean} timeStep A difference to next/previous step in minutes.
 */

/**
 * See `jQuery.ui.uiDatepicker` for details.
 *
 * @param {Object} [options] A map of additional options pass to the method.
 * @param {String} type The value respectively validation type must be 'date'.
 * @param {String} format A date formatter [look at the jQuery.i18n plugin].
 * @param {Integer} min A local timestamp (not UTC) as minimum valid date.
 * @param {Integer} max A local timestamp (not UTC) as maximum valid date.
 * @param {String} showOn A handle indication, how to show the datepicker. Supported handles: 'focus', 'button', 'both' (button and focus) or 'inline'.
 * @param {Integer,String} showOn A duration for show and hide effect. [look at jQuery.show() or jQuery.hide() for supported durations].
 * @param {String} show One off the supported show effects: 'show', 'fadeIn' or 'slideIn'.
 * @param {String} hide One off the supported hide effects: 'hide', 'fadeOut' or 'slideOut'.
 * @param {Array} weekDaysAvailable An array of 7 weekdays with indications wether a day is available. Week starts on sunday.
 * @param {Integer} numberOfMonths Set how many months to show at once.
 * @param {Boolean} showOtherMonth Display dates in other months (non-selectable) at the start or end of the current month. To make these days selectable use selectOtherMonths.
 * @param {Boolean} selectOtherMonth When true days in other months shown before or after the current month are selectable. This only applies if showOtherMonths is also true.
 * @param {Boolean} changeMonth Allows you to change the month by selecting from a drop-down list. You can enable this feature by setting the attribute to true.
 * @param {Boolean} changeYear Allows you to change the year by selecting from a drop-down list. You can enable this feature by setting the attribute to true. Use the yearRange option to control which years are made available for selection.
 * @param {Boolean} time Allows you to select time also. Requires a format which support time.
 * @param {Boolean} timeStep A difference to next/previous step in minutes.
 *
 * @method uiDatepicker
 * @member jQuery
 *
 * @since 6.11.0
 */

/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.datepicker
 */

define("ep/ui/datepicker", [
	"jquery",
	"ep",
	"$dict!ep/dict",
	"$tmpl!ep/ui/datepicker",

	"ep/date",
	"ep/sprite",
	"ep/fn/busy",
	"ep/ui/spinner"
], function ($, ep, epDict, tmplDatepicker) {

/*
 * @dictionary		ep.dict
 *
 * @translation		{Today}
 *					{Done}
 */

	$.widget('ui.uiDatepicker',$.ui.uiValidate,{
		isEventRegistered: false,

		options: {
	// ep.ui.Input
		//	big:				true,
		//	autofocus:			false,
		//	placeholder:		'text',
		//	info:				'text',

	// ep.ui.Validate
		//	accept:				'',
		//	min:				1,
		//	max:				20,
		//	minlength:			1,
		//	maxlength:			4,
		//	pattern:			[\w]+,
		//	required:			true,
		//	valid:				true,
		//	type:				'basic',
		//	format:				'',

	// ep.ui.Datepicker
		//	onAfterSetValue:	undefined					// callback called after value was set
		//	buttonBefore:	false,
			bubble: 			true,
			type:				'date',
			format:				'l',						// format for i18n
			showOn:				'button',					// show datepicker on 'focus' || 'button' || 'both' or 'inline'
			duration:			'normal',					// duration for show and hide effect
			show:				'fadeIn',					// show effect 'show' || 'fadeIn' || 'slideIn'
			hide:				'fadeOut',					// hide effect 'hide' || 'fadeOut' || 'slideOut'
			weekDaysAvailable:	[1,1,1,1,1,1,1],			// which days ar avaible boolean in array
			numberOfMonths:		1,							// display number of month
			showOtherMonth:		true,						// show days from other month
			selectOtherMonth:	false,						// days from other month ar clickable
			changeMonth:		true,						// enable select month
			changeYear:			true,						// enable select year
			region:				$.i18n.settings.region,		// region for i18n
			time:				true,						// use as date and time picker
			timeStep:			15,							// step for time spinner in minutes
			min:				undefined,					// min for range miliseconds || dateObj
			max:				undefined,					// max for range miliseconds || dateObj
			currentDate:		undefined,					// Sets the initially selected date.
															// Use this format: '{year: yyyy, month: mm, day: dd}'
			showTodayButton:	true,						// Show the 'Today' button.
			showDoneButton:		true,						// Show the 'X' / 'Complete' button.
			years: 				10,
			isBirthday:         false
		},

		_create: function(){
			this._superApply(arguments);
			this.open = false;

			this.unity = (this.options.unity || (ep.config.unity!==undefined && ep.config.unity===true)) ? true : false;

			// get instance of dict
			this.dict = epDict;

			var currentDate = this.options.currentDate,
				date;

			this.element.addClass('ep-datepicker-input');

			date = currentDate ? new ep.Date(currentDate.year, currentDate.month, currentDate.day) : new ep.Date();
			date
				.setMinutes(30)
				.setSeconds(0)
				.setMilliseconds(0);

			this._date =			date.clone().setHours(0,0,0,0);
			this._dateNow =			this._date.clone();
			this._dateSelected =	this._date.clone();
			this._timeSelected =	date.clone().setFullYear(0,0,0);

			this.domCache = $('<div>');
			this.cal = $.i18n.regions[ this.options.region ].calendar;

			this.elem.on( 'changeValue.uiDatepicker', $.proxy(this, '_changeElemVal') );

			this.showButton = $('<span>')
				.attr({
					'class':	'ep-uiInput ep-uiInput-base ep-uiInput-button ep-uiDatepicker-showButton'
				})
				.html(
					this.unity ? '<span class="fa fa-calendar fa-lg ep-uiInput-buttonSpriteOnly"></span>' : ep.sprite('calendar','s').addClass('ep-uiInput-buttonSpriteOnly')
				)
				.on( 'click.uiDatepicker', $.proxy(this, 'show') )
				.appendTo( this.domCache );

			this.stack.push( this.domCache[0], this.showButton[0] );

			this._changeElemVal();
		},

		_init: function(){
			var self = this,
				o = self.options;

			self._superApply(arguments);


			if( (/^(focus|both)$/).test(o.showOn) ){
				self.elem
					.off('focus.uiDatepicker')
					.on( 'focus.uiDatepicker', $.proxy(self, 'show') );
			}
			if( (/^(button|both)$/).test(o.showOn) ){
				if (o.buttonBefore) {
					self.wrapperSpan = $('<span />');
					self.wrapperSpan.insertBefore(self.placeholder || self.elem);
					self.wrapperSpan.append(self.elem);
					self.showButton.insertBefore(self.wrapperSpan);
				} else {
					self.showButton.insertAfter( self.placeholder || self.elem );
				}
			}
			else{
				self.showButton
					.appendTo( self.domCache );
			}

			if( self.container ){
				self._initCalCache();

				if( o.time && !self.timeBar ){
					self._createTimeBar();
				}
			}

			self._tooltipInit();

			if( o.showOn === 'inline' ){
				self.show();
			}
		},

		_initCalCache: function(){
			this._calCache = {
				weekDays: arrayRotate( $.merge([],this.cal.days.namesShort), this.cal.firstDay),
				months: $.merge([],this.cal.months.names)
			};
		},

		_tooltipInit: function(){
			this._superApply(arguments);
			if( this.tooltip && (/^(button|both)$/).test(this.options.showOn) ){
				this.tooltip.uiTooltip( 'option', 'offsetAdjust', [30,-1] );
			}
			else if( this.tooltip ){
				this.tooltip.uiTooltip( 'option', 'offsetAdjust', [3,-2] );
			}
		},

		_createBase: function(){
			var self = this;

			// the datepicker
			self.container = ep('<form>')
				.addClass('epDialog ep-uiDatepicker-container ui-dialog ui-front')
				.hide();

			// go to previous month
			self.prevButton = ep('<button type="button">')
				.append((self.unity ? $('<span class="fa fa-angle-left" />') : ep.sprite('arrow-l','s').addClass('ep-uiInput-buttonSpriteOnly')))
				.addClass('ep-uiDatepicker-prev')
				.on( 'click.uiDatepicker', function(){
					self._changeMonth(-1);
				});

			// go to next month
			self.nextButton = ep('<button type="button">')
				.append((self.unity ? $('<span class="fa fa-angle-right" />') : ep.sprite('arrow-r','s').addClass('ep-uiInput-buttonSpriteOnly')))
				.addClass('ep-uiDatepicker-next')
				.on( 'click.uiDatepicker', function(){
					self._changeMonth(1);
				});

				// bar on the bottom
				self.buttonBar = $('<div>')
					.addClass('ep-uiDatepicker-buttonBar')
					.appendTo( self.container );

			if (self.unity) {
				// additional unity classes
				self.container.addClass('ep-uiDatepicker-container-unity');
				self.buttonBar.addClass('ep-uiDatepicker-buttonBar-unity');

				// close the datepicker
				if (self.options.showDoneButton) {
					self.doneButton = ep('<button type="button">')
						.html('<span class="fa fa-remove" />')
						.appendTo( self.buttonBar )
						.addClass('ep-uiDatepicker-button-close')
						.on( 'click.uiDatepicker', $.proxy(self, 'hide') );
				}
			} else {
				// select the current day
				if (self.options.showTodayButton) {
					self.todayButton = ep('<button type="button">')
						.text( self.dict.translate('Today') )
						.appendTo( self.buttonBar )
						.addClass('ep-uiDatepicker-today')
						.on( 'click.uiDatepicker', function() {
							self._date = self._dateNow.clone().setHours(0,0,0,0);
							self._changeDay( self._date.getTime() );
							self._createCalendars();
						});
				}

				// close the datepicker
				if (self.options.showDoneButton) {
					self.doneButton = ep('<button type="button">')
					.text( self.dict.translate('Done') )
						.appendTo( self.buttonBar )
						.addClass('ep-uiDatepicker-done')
						.on( 'click.uiDatepicker', $.proxy(self, 'hide') );
				}
			}

			// live click handler to select a day on click
			self.container
				.on( 'click', 'a.ep-uiDatepicker-dayClick', $.proxy(self, '_selectDay') )
				.on( 'click', 'select.ep-uiDatepicker-yearSelect', function(evt) {
					evt.preventDefault();
					evt.stopPropagation();
				})
				.on( 'click', 'select.ep-uiDatepicker-monthSelect', function(evt) {
					evt.preventDefault();
					evt.stopPropagation();
				})
				.on( 'change', 'select.ep-uiDatepicker-monthSelect,select.ep-uiDatepicker-yearSelect', function() {
					self._changeMonth(parseInt(this.value,10));
				});

			// stack for calendars
			self.calendars = $();
			// reinit
			self._init();
			// add to DOM
			self.container.appendTo('body');
			// save in stack
			self.stack.push(
				self.container[0],
				self.prevButton[0],
				self.nextButton[0],
				self.buttonBar[0]
			);

			if (!self.unity && self.options.showTodayButton) {
				// save in stack
				self.stack.push(
					self.todayButton[0]
				);
			}

			if (self.options.showDoneButton) {
				self.stack.push(
					self.doneButton[0]
				);
			}
		},

		_createTimeBar: function(){
			this.timeBar = $('<div>')
				.addClass('ep-uiDatepicker-timeBar')
				.insertBefore( this.buttonBar );

			this.stack.push( this.timeBar[0] );

			this._createTimeSelect();
		},

		_createTimeSelect: function(){
			this.timeSelect = ep('<input>')
				.attr({
					'class': 'epWidth100',
					'placeholder': '00:00'
				})
				.appendTo( this.timeBar )
				.uiSpinner({
					type:		'date',
					step:		15,
					stepType:	'minutes',
					format:		't',
					region:		this.options.region
				})
				.uiInput();

			this.timeSelect
				.uiInput('addClass','ep-uiDatepicker-time')
				.on( 'changeValue.uiDatepicker', $.proxy(this, '_changeTimeVal') );

			this._setTimeVal();

			this.stack.push( this.timeSelect[0]	);
		},

		_createCalendars: function(){
			// set busy layer
			this.container.busy('show');
			// vars
			var self = this,
				o				= self.options,
				calendarsData	= self._calendarData(),
				calendars;
			// build weeks
			$.each( calendarsData, function( i, calendarData){
				while( calendarData.days.length >= 7 ){
					calendarData.weeks.push({days:calendarData.days.splice(0,7)});
				}
			});
			// build calendars
			calendars = tmplDatepicker(calendarsData);
			calendars
				.not(':first,:last')
				.addClass('ep-uiDatepicker-calendarMiddle');
			calendars
				.not(':last')
				.first()
				.addClass('ep-uiDatepicker-calendarFirst');
			calendars
				.not(':first')
				.last()
				.addClass('ep-uiDatepicker-calendarLast');
			// place prev button to new calendars and handle range
			self.prevButton
				.attr( 'disabled', !inRange( o.min, self._date.clone().setFirstMonthDay().setHours(0,0,0,-1).getTime() ) )
				.prependTo( calendars.filter(':first').find('.ep-uiDatepicker-titleBar') );
			// place next button to new calendars and handle range
			self.nextButton
				.attr( 'disabled', !inRange( undefined, self._date.clone().setLastMonthDay().setHours(0,0,0,0).addDate(1).getTime(), o.max ) )
				.appendTo( calendars.filter(':last').find('.ep-uiDatepicker-titleBar') );
			// remove old calendars
			self.calendars
				.html('')
				.remove();

			// show new calendars in datepicker
			self.container[self.unity ? 'append' : 'prepend'](calendars)
				.busy('hide');

			// move timebar behind calendar
			if (self.unity && self.timeBar) {
				self.timeBar.appendTo(self.container);
			}
			// set new calendars
			self.calendars = calendars;
		},

		_calendarData: function(){
			var o				= this.options,
				baseDate		= this._date.clone().addMonth( - Math.ceil(o.numberOfMonths/2) ),
				nowTime			= this._dateNow.getTime(),
				selectedTime	= this._dateSelected.getTime(),
				region			= {region: this.options.region},
				baseMonth		= this._date.getMonth(),
				changeMonth		= null,
				changeYear		= null,
				data			= [],
				i;

			if( o.changeMonth ){
				changeMonth = [];
				$.each( this._calCache.months, function( i, name ){
					if( name ){
						changeMonth.push({ value: (baseMonth-i)*-1, name: name, currentMonth: baseMonth===i });
					}
				});
			}

			if( o.changeYear ){
				var yDate = this._date.clone();

				changeYear = [{ value: 0, name: yDate.getFormat( "yyyy", region ), currentYear: true }];

				for( i=1 ; i<=o.years ; i++ ){
					yDate.addFullYear(-1);
					changeYear.unshift({ value: i*-12, name: yDate.getFormat( "yyyy", region ), currentYear: false });
				}

				if(o.isBirthday === true){
					yDate = this._date.clone();

					for( i=1 ; i<= o.years, yDate.date.getFullYear() < this._dateNow.date.getFullYear() ; i++ ){
					yDate.addFullYear(1);
					changeYear.push({ value: i*12, name: yDate.getFormat( "yyyy", region ), currentYear: false });
					}

				} else {
					yDate.addFullYear(o.years);

					for( i=1 ; i<=o.years ; i++ ){
					yDate.addFullYear(1);
					changeYear.push({ value: i*12, name: yDate.getFormat( "yyyy", region ), currentYear: false });
					}

				}
			}


			for( i=0,iLength=o.numberOfMonths ; i<iLength ; i++ ){
				var date =			baseDate.addMonth(1),
					iDate =			date.clone().setDate(0),
					iFirstDate =	date.clone().setFirstMonthDay(),
					iFirstTime =	iFirstDate.getTime(),
					iLastTime =		date.clone().setLastMonthDay().getTime(),
					iDays =			iFirstDate.getMonthLength(),
					iWeekDay =		iFirstDate.getDay() - this.cal.firstDay,
					add =			{
										year:		iFirstDate.getFormat( "yyyy", region ),
										years:		changeYear,
										maxYear:	o.max ? new Date(o.max).getYear() + 1900 : Number.MAX_SAFE_INTEGER,
										minYear:	o.min ? new Date(o.min).getYear() + 1900 : 0,
										month:		iFirstDate.getFormat( "MMMM", region ),
										months:		changeMonth,
										weekDays:	this._calCache.weekDays,
										weeks:		[],
										days:		[]
									};

				if( iWeekDay<0 ){
					iWeekDay = iWeekDay+7;
				}

				iDate.addDate( - iWeekDay );

				for( var j=1,jLength=(Math.ceil( (iWeekDay+iDays) / 7) * 7 ) ; j<=jLength ; j++ ){
					var jDate =			iDate.addDate(1),
						jTime =			jDate.getTime(),
						currentMonth =	inRange( iFirstTime, jTime, iLastTime ),
						clickable =		inRange( o.min, jTime, o.max ) && o.weekDaysAvailable[ jDate.getDay() ];

					add.days.push({
						'date':				jDate.clone(),
						'tstamp':			jTime,
						'day':				jDate.getDate(),
						'currentMonth':		!!currentMonth,
						'clickable':		(currentMonth || o.selectOtherMonth) && clickable,
						'visible':			!!(currentMonth || o.selectOtherMonth || o.showOtherMonth),
						'selected':			jTime===selectedTime,
						'now':				jTime===nowTime
					});
				}

				data.push(add);
			}

			return data;
		},

		_setElemVal: function(settings) {
			var self = this,
				o = self.options,
				val = new ep.Date(
							self._dateSelected.getFullYear(),
							self._dateSelected.getMonth(),
							self._dateSelected.getDate(),
							isNaN(self._timeSelected.getHours()) ? 0 : self._timeSelected.getHours(),
							isNaN(self._timeSelected.getMinutes()) ? 0 : self._timeSelected.getMinutes(),
							isNaN(self._timeSelected.getSeconds()) ? 0 : self._timeSelected.getSeconds()
						)
						.getFormat(
							o.format,
							{region: o.region}
						),
				closePicker;

			// Strange case: datepicker only works with attr('value', ...) although elem is a form element
			// if(settings && (settings.noBubbling !== undefined) && (settings.noBubbling === true)){
			// 	self.elem.val( val );
			// }
			// else{
			// 	self.elem.attr( 'value', val );
			// }

			if(o.bubble){
				self.elem.val( val );
			}
			self.elem.attr( 'value', val );

			// callback called after value was set
			if(o.onAfterSetValue !== undefined){
				o.onAfterSetValue(self.elem);
			}

			// close date picker after a date was picked (UNITY-3413)
			closePicker = (settings && settings.keepOpen !== undefined) ? false : true;
			if (self.unity && closePicker) {
				self.hide();
			}
		},

		_getElemVal: function(){
			var date = $.i18n.parseDate( this.elem.val(), this.options.format, {region:this.options.region} );
			return date ? new ep.Date( date.getTime() ) : null;
		},

		_changeElemVal: function(){
			var o = this.options,
				date = this._getElemVal();

			if( date && inRange( o.min, date.clone().setLastMonthDay().getTime() ) && inRange( undefined, date.clone().setFirstMonthDay().getTime(), o.max ) ){
				if( this.elem.is(':focus') ){
					this._date = date.clone().setHours(0,0,0,0);
				}
				this._dateSelected = date.clone().setHours(0,0,0,0);
				this._timeSelected = date.clone().setFullYear(0,0,0);

				if( this.timeSelect ){
					this._setTimeVal();
				}
			}
			if( this.container && this.container.filter('.ep-uiDatepicker-calendar').length === 0 ){
				this._createCalendars();
			}
		},

		_setTimeVal: function(){
			if( !this.timeSelect.is(':focus') ){
				this.timeSelect.val( this._timeSelected.getFormat( 't', {region:this.options.region} ) );
			}
		},

		_getTimeVal: function(){
			var date = $.i18n.parseDate( this.timeSelect.val(), 't', {region:this.options.region} );
			return date ? new ep.Date( date.getTime() ) : null;
		},

		_changeTimeVal: function(event){
			var date = this._getTimeVal();

			if( date ){
				this._timeSelected = date.setFullYear(0,0,0);
				this._setElemVal({keepOpen: true});
			}
		},

		_changeMonth: function( inc ){
			this._date.addMonth(inc);
			this._createCalendars();
		},

		_changeDay: function( tstamp ){
			this._dateSelected = new ep.Date(tstamp);
			this._setElemVal();
		},

		_selectDay: function( event ){
			this._changeDay( parseInt( $(event.target).attr('data-tstamp'), 10 ) );
		},

		_moveToTop: ep.ui.createMoveToTop('container'),

		_show: function(){
			var self = this,
				o = self.options,
				elem = self.element,
				datePicker = ep('.ep-uiDatepicker-calendar').closest('form'),
				uiDatePicker = ep(':ui-UiDatepicker');

			datePicker.each(function(index, item) {
				// if current item is visible => close datepicker
				if ($(item).is(':visible')) {
					$(item).trigger('closeDatePicker');
				}
			});

			uiDatePicker.each(function(index, item) {
				if ((item !== elem[0]) && uiDatePicker.eq(index).uiDatepicker('isOpen')) {
					uiDatePicker.eq(index).uiDatepicker('hide');
				}
			});

			// register click event (namespace = datePicker) if it is not registered yet
			if (!self.isEventRegistered) {
				$(document).on('click' + self.eventNamespace, function (e) {
					var datePicker = ep('.ep-uiDatepicker-calendar').closest('form'),
						onButton = (e.target === self.showButton[0]) ? true : self.showButton[0].contains(e.target),
						boundingClientRect = self.container[0].getBoundingClientRect();

					datePicker.each(function(index, item) {
						// Move to top, but keep silent
						self._moveToTop({}, true);
					});

					// check if click is inside the currently open datepicker => if not, close
					if (!(boundingClientRect.top < e.clientY && e.clientY < boundingClientRect.bottom && boundingClientRect.left < e.clientX && e.clientX < boundingClientRect.right) && !onButton) {
						self.hide();
					}
				});

				self.isEventRegistered = true;
			}

			if( o.showOn !== 'inline' ){
				this.container.contextOrientation( this.elem, 'bottom', [-1,1] ).focus();
				this.container
					.stop(true,true)
					[ $.fn[o.show] ? o.show : 'show' ](o.duration);

				$('html').on( 'keyup', $.proxy(this,'_hide') );
			}
			else{
				this.buttonBar
					.hide();
				this.container
					.insertAfter(this.elem)
					.css('position','relative')
					.show();
			}

			this.open = true;
			this.elem.blur();
		},

		show: function(){
			var self = this;

			if( this.elem.is(":disabled") ){
				return;
			}
			if( !this.container ){
				this._createBase();
			}

			this._show();
			this._changeElemVal();
			this._setValidStatus(true);
		},

		_hide: function( event ){
			if( event.keyCode===27 ){
				this.hide();
				this._validate(event);
			}

			this.open = false;
		},

		hide: function(){
			var self = this,
				o = self.options;

			$(document).off('click' + self.eventNamespace);
			self.isEventRegistered = false;

			if( o.showOn !== 'inline' ){
				self.container
					.stop(true,true)
					[ $.fn[o.hide] ? o.hide : 'hide' ](o.duration);

				$('html').off( 'keyup', $.proxy(self,'_hide') );
			}
		},

		/**
		 * Returns the current date for the datepicker as `ep.Date()` object.
		 *
		 * @method getDate
		 * @member jQuery.ui.uiDatepicker
		 *
		 * @since 6.11.0
		 */
		getDate: function(){
			return new ep.Date(
							this._dateSelected.getFullYear(),
							this._dateSelected.getMonth(),
							this._dateSelected.getDate(),
							this._timeSelected.getHours(),
							this._timeSelected.getMinutes(),
							this._timeSelected.getSeconds()
						);
		},

		isOpen: function () {
			return this.open;
		}
	});

	var arrayRotate = function( array, size ){
			var tmp = array.splice( 0, size );

			return array.concat( tmp );
		},

		inRange = function( min, val, max ){
			min = new ep.Date(typeof min === 'object' && min.getTime ? min.getTime() : min).setHours(0,0,0,0).getTime();
			max = new ep.Date(typeof max === 'object' && max.getTime ? max.getTime() : max).setHours(0,0,0,0).getTime();
			return !min || val >= min ? !max || val <= max ? true : false : false;
		};

	return ep;

});
