(function () {
    'use strict';
    var scribePluginPlaceholder = function (toolbarNode, options) {
       var dropdown = toolbarNode.querySelector(options.selector);

        /**
         * toggleDropdownHandler for toggling the dropdown menu
         *
         * @param {Event} event event object
         */
        var toggleDropdownHandler = function () {
            //gSelection = new scribe.api.Selection();
            dropdown.classList.toggle('open');
            if (dropdown.classList.contains('open')) {
                document.addEventListener('click', cancelHandler);
            } else {
                document.removeEventListener('click', cancelHandler);
            }
        };

        /**
         * cancelHandler for closing dropdown when clicked
         * somewhere outside the toolbar
         *
         * @param {Event} event event object
         */

        var cancelHandler = function (event) {
            if (!dropdown.contains(event.target)) {
                toggleDropdownHandler();
            }
        };

        return function (scribe) {
            // if a placeholder dropdown exists
            if (!dropdown) {
                return;
            }

            // Create own simulated command for the dropdown-toggle
            var commandNodes = dropdown.querySelectorAll('[data-command-name]'),

                /**
                 * replace a selection/range with a html fragment
                 *
                 * @param {scribe} object - reference to scribe editor
                 * @param {content} string - html string with the new content
                 */
                replaceSelectionWithFragment = function(scribe, content) {
                    var scribeSelection,
                        selection,
                        range,
                        fragment,
                        fragmentRoot,
                        child;

                    // get selection via scribe instance
                    scribeSelection = new scribe.api.Selection();
                    selection = scribeSelection.selection;

                    // check if Selection object contains at least one range
                    if (selection.getRangeAt && selection.rangeCount) {
                        // get the first range and remove it
                        range = selection.getRangeAt(0);
                        range.deleteContents();

                        // check if browser supports range.createContextualFragment method
                        if (range.createContextualFragment) {
                            // create html fragment from content
                            fragment = range.createContextualFragment(content);

                        // workaround for non supporting browsers
                        } else {
                            // create html fragment from content
                            fragmentRoot = document.createElement("div");
                            fragmentRoot.innerHTML = content;
                            fragment = document.createDocumentFragment();

                            // get first child of fragment root
                            child = fragmentRoot.firstChild;

                            while (child) {
                                // append child to fragment (means that child gets removed from fragmentRoot)
                                fragment.appendChild(child);
                                child = fragmentRoot.firstChild;
                            }
                        }

                        // insert new fragment as range
                        range.insertNode(fragment);
                    }
                },

                // Register commands with their respective command action
                registerCommands = function (scribe, commandName, placeholder) {
                    var command = new scribe.api.Command(commandName);

                    command.execute = function () {
                        // insert new html fragment
                        replaceSelectionWithFragment(scribe, placeholder);

                        scribe.el.focus();
                        getSelection().collapseToEnd();
                    };

                    command.queryEnabled = function () {
                        return true;
                    };

                    return command;
                };

            // Get scribe-commands for the dropdown with their respective command action
            var commands = [].map.call(commandNodes, function (node) {
                return {
                    command: (node.dataset.commandName === "dropdownToggle") ? scribe.getCommand(node.dataset.commandName) : registerCommands(scribe, node.dataset.commandName, node.dataset.placeholderValue),
                    title: node.textContent,
                    name: node.dataset.commandName
                };
            });

            // Add EventListener to dropdown field to toggle
            dropdown.addEventListener('click', toggleDropdownHandler);

            // Register commands on scribe object
            commands.forEach(function (command, index) {
                scribe.commands[command.name] = command.command;
            });
        };
    };

    // Module system magic dance
    if (typeof module !== 'undefined' && typeof module.exports === 'object') {
        module.exports = scribePluginPlaceholder;
    } else if (typeof define === 'function' && typeof define.amd === 'object') {
        define(function () {
            return scribePluginPlaceholder;
        });
    } else {
        window.scribePluginPlaceholder = scribePluginPlaceholder;
    }
}());